@@ -3,7 +3,7 @@ package Hades;
 use 5.006;
 use strict;
 use warnings;
-our $VERSION = '0.06';
+our $VERSION = '0.07';
 use Module::Generate;
 use Switch::Again qw/switch/;
 
@@ -17,7 +17,12 @@ sub run {
 	$args->{eval} = _read_file($args->{file}) if $args->{file};
 	my $mg = Module::Generate->start;
 	$args->{$_} && $mg->$_($args->{$_}) for (qw/dist lib tlib author email version/);
+	if ($args->{realm}) {
+		$class = sprintf "Hades::Realm::%s", $args->{realm};
+		eval "require $class";
+	}
 	my $self = $class->new($args);
+	$self->can('module_generate') && $self->module_generate($mg, $class);
 	my ($index, $ident, @lines, @line, @innerline, $nested) = (0, '');
 	while ($index < length $self->{eval}) {
 		my $first_char = $self->index($index++);
@@ -162,6 +167,11 @@ sub run {
 													$value = '"' . $value . '"'
 														if $value !~ m/^(\{|\[|\"|\'|q)|(\d+)/;
 													$meta{$name}->{default} =  $value;
+													if ($meta{$name}->{params_map}) {
+														$meta{$name}->{params_map}->{
+															$meta{$name}->{param}->[-1]
+														}->{default} = $value;
+													}
 												},
 												qr/^(\:private|\:p)$/ => sub {
 													$meta{$name}->{private} = 1;
@@ -298,6 +308,8 @@ sub build_sub {
 			$params_explanation .= ', ' if $params_explanation;
 			$params .= ', ' .  $param;
 			my $pm = $meta->{$name}->{params_map}->{$param};
+			$subtype .= qq|$param = defined $param ? $param : $pm->{default};|
+				if ($pm->{default});
 			$subtype .= $self->build_coerce($name, $param, $pm->{coerce});
 			if ($pm->{type}) {
 				my $error_message = ($pm->{type} !~ m/^(Optional|Any|Item)/
@@ -323,6 +335,7 @@ sub build_sub {
 	}|;
 	$params =~ s/^,\s*//;
 	my $example = qq|\$obj->$name($params)|;
+
 	$mg->sub($name)->code($code)
 		->pod(qq|call $name method. Expects $params_explanation.|)
 		->example($example)
@@ -653,8 +666,8 @@ sub build_test_data {
 			my ($val, @matches) = @_;
 			$matches[1] = '"' . $matches[1] . '"' if $matches[1] =~ m/^[a-zA-Z]/;
 			return (
-				qq|bless { test => 'test' }, $matches[1]|,
-				qq|bless { test => 'test' }, $matches[1] . 'Error'|,
+				qq|bless({ test => 'test' }, $matches[1])|,
+				qq|bless({ test => 'test' }, $matches[1] . 'Error')|,
 				$self->_generate_test_string
 			);
 		},
@@ -665,8 +678,8 @@ sub build_test_data {
 			my ($val, @matches) = @_;
 			$matches[1] = '"' . $matches[1] . '"' if $matches[1] =~ m/^[a-zA-Z]/;
 			return (
-				qq|bless \"", $matches[1]|,
-				qq|bless \"", $matches[1] . 'Error'|,
+				qq|bless( \"", $matches[1])|,
+				qq|bless( \"", $matches[1] . 'Error')|,
 				$self->_generate_test_string,
 				q|{}|
 			);
@@ -740,7 +753,7 @@ sub build_test_data {
 		},
 		qr/^(Object)$/ => sub {
 			return (
-				q|bless {}, 'Test'|,
+				q|bless({}, 'Test')|,
 				q|[]|,
 				$self->_generate_test_string
 			);
@@ -786,7 +799,7 @@ sub build_test_data {
 			my %map;
 			while (@matches) {
 				my ($match) = (shift @matches);
-				if ($match =~ m/(Map|Tuple|ArrayRef|Dict)\[/) {
+				if (@matches && $match =~ m/(Map|Tuple|ArrayRef|Dict)\[/) {
 					my $lame = sub {
 						my $copy = shift;
 						while ($copy =~ s/\[[^\[\]]+\]//g) {}
@@ -795,6 +808,7 @@ sub build_test_data {
 					while ($lame->($match .=  ', ' . shift @matches)) {}
 				}
 				my ($k, $v) = map { my $h = $_; $h =~ s/^\s*|\s*$//g; $h; } split('=>', $match, 2);
+				$v =~ s/,\s*$//;
 				my @values = $self->build_test_data($v, $name, $required);
 				push @values, 'undef' unless $v =~ m/^Optional/;
 				$map{$k} = \@values;
@@ -906,7 +920,7 @@ sub build_tests {
 	} : $meta->{meta} eq 'ACCESSOR' ? do {
 		push @tests, ['can_ok', qq|\$obj|, qq|'$name'|];
 		$meta->{private} ? do {
-			push @tests, ['eval', qq|\$obj->name|, 'private method'];
+			push @tests, ['eval', qq|\$obj->$name|, 'private method'];
 		} : do {
 			push @tests, ['is', qq|\$obj->$name|, 'undef'] if !$meta->{required} && !$meta->{default};
 			my (@test_cases) = $self->build_test_data($meta->{type}->[0] || 'Any', $name);
@@ -964,7 +978,7 @@ Hades - Less is more, more is less!
 
 =head1 VERSION
 
-Version 0.06
+Version 0.07
 
 =cut
 
@@ -1087,6 +1101,10 @@ The version number of the distribution/module.
 
 =back
 
+=item realm
+
+The Hades realm that is used to generate the code.
+
 =cut
 
 =head1 Hades
